#ifndef _CONO_MAP_H_
#define _CONO_MAP_H_

#include <gtk/gtk.h> 
typedef enum {Uniaxial, Biaxial} AxialType;
typedef enum {Single, Circular, Animation} DrawingKind;
typedef enum {In, Out} TestPlate;
typedef enum {WithinOpeningAngle, WholeAngle} DrawingRange ;
typedef enum {RealProjection, TanProjection, WulffProjection} ProjectionMethod; 

typedef struct 
{
   AxialType axial_type;
   DrawingKind drawing_kind;
   TestPlate test_plate;
   DrawingRange drawing_range;
   ProjectionMethod projection;
} DrawingMethod;

typedef enum {Done, Yet} Setting;

typedef struct 
{
  gfloat image_radius_for_Single;      /* in dot */ 
  gfloat image_radius_for_Circular;    /* in dot */ 
  gfloat image_radius_for_Animation;   /* in dot */
  gfloat image_radius;
  gfloat image_size_for_Single;      /* in dot */ 
  gfloat image_size_for_Circular;    /* in dot */ 
  gfloat image_size_for_Animation;   /* in dot */
  gfloat image_size;
  gfloat image_num_for_Circular;  
  gfloat image_num_for_Animation;
  gfloat image_num;
  gfloat sampling_step_for_Circular;
  gfloat sampling_step_for_Animation;
  gfloat sampling_step;
  gfloat thickness;                  /* of thin-section (in micron)*/
  gfloat omega, epsilon;             /* Principal RI  for Uniaxial Body*/
  gfloat alpha, beta, gamma;         /* Principal RI  for Biaxial Body*/
  gfloat RI;                         /* average RI */
  gfloat optic_axial_angle;          /* For Biaxial Body*/
  gfloat NA;                         /* Numerical Apperture*/
  gfloat c_theta, c_phi;             /* C axis Orientation for Uniaxial Body*/
  gfloat x_theta, x_phi;             /* X axis Orientation for Biaxial Body*/
  gfloat y_theta, y_phi;             /* Y axis Orientation for Biaxial Body*/
  gfloat z_theta, z_phi;             /* Z axis Orientation for Biaxial Body*/
  gfloat a_theta, a_phi;             /* A axis Orientation for Biaxial Body*/
  gfloat b_theta, b_phi;             /* B axis Orientation for Biaxial Body*/
  Setting setting_uniaxis, setting_biaxis;
} Parameter;

typedef struct
{
   double th, ph, x, y;
} OpticAxisOrientation;


extern DrawingMethod drawing_method;
extern  Parameter parameter;
extern OpticAxisOrientation Aaxis, Baxis, Caxis;
  
#endif
