/*===========================================================================*
 * main.c                                                                    *
 *                                                                           *
 *      Main routine for cono_map                                            *
 *                   version 2.0.0                                           *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *      int main(int ac, char*[])                                            *
 *                                                            (by A. Miyake) *
 *===========================================================================*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif  

#include <gtk/gtk.h>
#include <math.h>

#include "intl.h"
#include "cono_map.h"


/*==================*
 * GLOBAL VARIABLES *   
 *==================*/
DrawingMethod drawing_method;
Parameter parameter;
GtkStyle *GreekStyle;
GtkTooltips *tooltips;

/*====================*
 * EXTERNAL FUNCTIONS *   
 *====================*/
extern GtkWidget* MainWindow();  /* from main_window.c */

/*====================*
 * Internal Procedure *
 *====================*/
static void paramet_init();
static void setGreekStyle();

int
main(int ac, char* av[])
{
#ifdef ENABLE_NLS
  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);
#endif

   gtk_set_locale();
   
   gtk_init(&ac, &av); 

   gdk_rgb_init();

   /* initializing drawing_method and parameter */
   paramet_init();
   setGreekStyle();
   tooltips = gtk_tooltips_new ();
   
   gtk_widget_show_all (MainWindow());

   gtk_main();
   return 0;
}

/*======================================================================*
 * Internal Procedure                                                   *
 *                                                                      *
 *                     paramet_init                                     *
 *                     setGreekStyle                                    *
 *======================================================================*/

static void 
paramet_init()
{
   parameter.image_radius_for_Single = 100;
   parameter.image_radius_for_Circular = 60;
   parameter.image_radius_for_Animation = 50;

   parameter.thickness = 30;
   parameter.omega = 1.544;
   parameter.epsilon = 1.553;
   parameter.alpha = 1.552;
   parameter.gamma = 1.587;
   parameter.optic_axial_angle = 30; 
   parameter.NA = 0.65;
   parameter.c_phi = parameter.c_theta = 0;
   parameter.x_phi = 0; parameter.x_theta = 90;
   parameter.y_phi = 90; parameter.y_theta = 90;
   parameter.z_phi =  0; parameter.z_theta = 0;
   parameter.a_phi =  0; parameter.a_theta = 15;
   parameter.b_phi = 180; parameter.b_theta = 15;
   parameter.image_size_for_Single = 200;
   parameter.image_size_for_Circular = 120;
   parameter.image_size_for_Animation = 100;
   parameter.sampling_step_for_Circular = 30;
   parameter.sampling_step_for_Animation = 5;
}	

static void 
setGreekStyle()
{
   GtkStyle *style;
   GdkFont *font = gdk_font_load("-*-symbol-*-*-*-*-16-*-*-*-*-*-*-*");
   style = gtk_widget_get_default_style();
   GreekStyle = gtk_style_copy(style);
   GreekStyle->font = font;
}   
