/*===========================================================================*
 * save_image.c                                                              *
 *                                                                           *
 *      Procedures saving image data                                         *
 *                                                                           *
 *      gchar* get_file_extention(gchar* file)                               *
 *      void save_image(gchar* file, gchar* ext)                             *
 *                                                            (by A. Miyake) *
 *===========================================================================*/
#include <stdio.h>
#include <jpeglib.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include "intl.h"

#include <gtk/gtk.h>
#include "cono_map.h"
#include "PBar.h"
#include "RGB_graphics.h"

/*===================*
 * EXTENAL VARIABLES *   
 *===================*/
extern guchar *RGB24;
extern guchar image_color_mod[256];
extern gint current_image_index;
extern gint image_num;
extern gint image_size, total_image_size, mpeg_image_size;
extern gfloat image_radius;
extern float RI, NA;
extern gfloat FrameRate;

/*====================* 
 * EXTERNAL FUNCTIONS *
 *====================*/
extern double (*get_dis)(double);   
extern int mpeg_encode(char* paramet_file);
extern void get_nth_BiaxialAxes(int nth);
extern void get_nth_UniaxialAxis(int nth);

/*====================* 
 * INTERNAL FUNCTIONS *
 *====================*/
static int save_as_jpg (char *file, unsigned char *RGB240, int width, 
			int height);
static int save_as_ppm(char* file, unsigned char* RGB240, int width, 
		       int height);
static int (*file_saver)(char* file, unsigned char* RGB240, int width, 
		       int height);

static void draw_opticaxes_and_circle(unsigned char* RGB240, gint size, 
				      gint index);

static int save_as_mpg(char* file );
static char* create_paramet_file( char* mpg_file, char* pnm_group,int pnm_num);
static int pnmstompg(char* mpeg_file_name, char* pnm_group, int pnm_num);
static void clean_pnm_files(gchar* pnm_group, int num);      
static void image_expand(unsigned char *RGB240, unsigned char* new_RGB240);


/*===========================================================================*
 * (EXPORTED PROCEDURES)                                                     *
 *                         get_file_extention                                *
 *===========================================================================*/
gchar* 
get_file_extention(gchar* file)
{
   gchar *f, *ext;

   if((f=strrchr(file, '/')) == NULL) f = file;
   
   if((ext=strrchr(file, '.')) == NULL) ext = NULL;
   else      ext++;
   return ext;
}      

   
/*===========================================================================*
 * (EXPORTED PROCEDURES)                                                     *
 *                         save_image                                        *
 *                                                                           *
 *   If jpg or ppm file is selected, image is obtained from ImageArea (X)    *
 *   if mpg file is selected, image is obtained from RGB24 (RAM)             *
 *===========================================================================*/
void
save_image(gchar* file, gchar* ext)  
{
   guchar* RGB_DATA;
   gint data_size;
   gint k; 

   if( strcmp(ext, "mpg") == 0 )
     save_as_mpg(file);
   else
   {
      data_size = 3*image_size*image_size; 
      RGB_DATA = (guchar *)g_malloc(sizeof(guchar)*data_size);

      for(k = 0; k < data_size; k++)
	RGB_DATA[k] = image_color_mod[RGB24[current_image_index*data_size+k]];
 
      if(drawing_method.drawing_range == WholeAngle)
	draw_opticaxes_and_circle(RGB_DATA, total_image_size, 
				  current_image_index);

      if( strcmp(ext, "ppm") == 0 )
	file_saver = &save_as_ppm;
      else if( strcmp(ext, "jpg") == 0 )
	file_saver = &save_as_jpg;
  
      file_saver(file, RGB_DATA, image_size, image_size);
      g_free(RGB_DATA);
   }
}

/*===========================================================================*
 * (INTERNAL PROCEDURE)                                                      *
 *                         draw_opticaxes_and_circle                         *
 *===========================================================================*/
static void 
draw_opticaxes_and_circle(unsigned char* RGB240, gint size, gint index)
{
   double ax, bx, cx, ay, by, cy, x0, y0;
   int visible_radius;
   double R;
   int n,conoscopic_num;

   if(drawing_method.drawing_kind == Circular)
   {
      visible_radius = rint(parameter.image_size/2.0*get_dis(asin(NA / RI)));
      R = (total_image_size-parameter.image_size)/2;
      conoscopic_num = parameter.image_num;
   }
   else
   {
      visible_radius = rint(size/2.0*get_dis(asin(NA / RI)));
      R = 0;
      conoscopic_num = 1;
   }
     
   for(n = 0; n < conoscopic_num; n++)
   {
      if(drawing_method.drawing_kind == Circular)
      {
	 x0 = R*cos(2*M_PI*n/parameter.image_num) + R + image_radius - 0.5;
	 y0 = -R*sin(2*M_PI*n/parameter.image_num)+ R + image_radius - 0.5;
      }
      else
      {
	 x0 = (size - 1)/2.0;
	 y0 = (size - 1)/2.0;
      }

      RGB_circle(RGB240, size, size, 0, 255, 0, rint(x0), rint(y0), 
		 visible_radius);
      if(drawing_method.drawing_kind == Circular)
      {
	 get_nth_UniaxialAxis(n);
	 get_nth_BiaxialAxes(n);
      }
      else
      {
	 get_nth_UniaxialAxis(index);
	 get_nth_BiaxialAxes(index);
      }

      if(drawing_method.axial_type == Uniaxial)
      {
	 cx = x0 + size/2 * Caxis.x;
	 cy = y0 - size/2 * Caxis.y;
	 RGB_plot(RGB240, size, size, 255, 255, 0, cx, cy, 2.0);
      }
      else
      {
	 ax = x0 + size/2 * Aaxis.x;
	 ay = y0 - size/2 * Aaxis.y;
	 bx = x0 + size/2 * Baxis.x;
	 by = y0 - size/2 * Baxis.y;
	 RGB_plot(RGB240, size, size, 255, 255, 0, ax, ay, 2.0);
	 RGB_plot(RGB240, size, size, 255, 255, 0, bx, by, 2.0);
      }
   }
}

/*===========================================================================*
 * (INTERNAL PROCEDURE)                                                      *
 *                         save_as_jpg                                       *
 *===========================================================================*/
static int
save_as_jpg (char *file, unsigned char *RGB240, int width, int height)
{
      struct jpeg_compress_struct cinfo;
      struct jpeg_error_mgr jerr;
      JSAMPROW            row_pointer[1];
      int                 row_stride;
      FILE *f;
	     
      f = fopen(file, "wb");
      if (f)
	{
	  cinfo.err = jpeg_std_error(&jerr);
	  jpeg_create_compress(&cinfo);
	  jpeg_stdio_dest(&cinfo, f);
	  cinfo.image_width = width;
	  cinfo.image_height = height;
	  cinfo.input_components = 3;
	  cinfo.in_color_space = JCS_RGB;
	  jpeg_set_defaults(&cinfo);
	  jpeg_start_compress(&cinfo, TRUE);
	  row_stride = cinfo.image_width * 3;
	  while (cinfo.next_scanline < cinfo.image_height)
	    {
	      row_pointer[0] = RGB240 + (cinfo.next_scanline * row_stride);
	      jpeg_write_scanlines(&cinfo, row_pointer, 1);
	    }
	  jpeg_finish_compress(&cinfo);
	  jpeg_destroy_compress(&cinfo);
	  fclose(f);
	  return 1;
	}
      return 0;
}

/*===========================================================================*
 * (INTERNAL PROCEDURE)                                                      *
 *                         save_as_ppm                                       *
 *===========================================================================*/
static int 
save_as_ppm(char* file, unsigned char* RGB240, int width, int height)
{
   FILE* fp = fopen( file, "wb" );
   if(fp == 0)
     return 0;

   fprintf( fp, "P6 %d %d %d\n", width, height, 255 );
   fwrite(RGB240, sizeof(guchar), 3*width*height, fp); 
   fclose(fp);
   return 1;
}

/*===========================================================================*
 * (INTERNAL PROCEDURE)                                                      *
 *                         save_as_mpg                                       *
 *===========================================================================*/
static int 
save_as_mpg(char* file )
{
   guchar* RGB240;
   guchar* new_RGB240;
   char pnm_file[100];
   char *tmp_mpg = "_conomap_tmp.mpg";
   char *pnm_group = "_conomap_pnm_tmp";
   int k, n;
   int data_size = 3*image_size*image_size;

   show_PBar(_("Expanding Images"));
   for(n = 0; n < image_num; n++)
   {
      if( update_PBar((float)(n+1)/image_num) == FALSE) 
      {
	 clean_pnm_files(pnm_group, n);
	 return 0;
      }
 
      RGB240 = (guchar *)g_malloc(sizeof(guchar)*data_size);
      for(k = 0; k < data_size; k++)
	RGB240[k] = image_color_mod[RGB24[n*data_size+k]];

      new_RGB240 = 
	(guchar*)g_malloc(sizeof(guchar)*3*mpeg_image_size*mpeg_image_size);
      image_expand(RGB240, new_RGB240);

      if(drawing_method.drawing_range == WholeAngle)
	draw_opticaxes_and_circle(new_RGB240, mpeg_image_size, n);

      sprintf(pnm_file, "%s%d.pnm", pnm_group, n);
      save_as_ppm(pnm_file, new_RGB240, mpeg_image_size, mpeg_image_size);
      
      g_free(new_RGB240);
      g_free(RGB240);
   }

   show_PBar(_("Creating MPEG File")); 
   if(pnmstompg(tmp_mpg, pnm_group, image_num)==0)
   {
      clean_pnm_files(pnm_group, image_num);
      remove(tmp_mpg);
      return 0;
   }
   else
   {
      remove(file);
      rename(tmp_mpg, file);
      clean_pnm_files(pnm_group, image_num);
      return 1;
   }
   g_free(RGB24);
}

/*===========================================================================*
 * (INTERNAL PROCEDURE)                                                      *
 *                                                                           *
 *  The following functions are for creating mpeg_file                       *
 *                                                                           *
 *               create_paramet_file                                         *
 *               pnmstompg                                                   *
 *               clean_pnm_files                                             *
 *               image_expand                                                *
 *===========================================================================*/
static char* 
create_paramet_file( char* mpg_file, char* pnm_group, int pnm_num)
{
   FILE *fp;
   static char mpeg_paramet[] = "_conomap_mpeg.paramet";
   int MpegFrameRate = 24;
   int i, k;

   fp = fopen(mpeg_paramet, "wb");
   fprintf(fp, "PATTERN         IBBPBBPBBPBBPBB\n");
   fprintf(fp, "OUTPUT          %s\n", mpg_file);
   fprintf(fp, "BASE_FILE_FORMAT        PNM\n");
   fprintf(fp, "INPUT_CONVERT    *\n");
   fprintf(fp, "GOP_SIZE        30\n");
   fprintf(fp, "SLICES_PER_FRAME        1\n");
   fprintf(fp, "INPUT_DIR       .\n");
   fprintf(fp, "INPUT\n");

   i = 0; k = 0;
   while( i < pnm_num)
   {
      fprintf(fp, "%s%d.pnm\n", pnm_group, i);
      k++;
      while( (float)i/FrameRate < (float)k/MpegFrameRate) i++;
   }

   fprintf(fp, "END_INPUT\n");
   fprintf(fp, "PIXEL           HALF\n");
   fprintf(fp, "RANGE           10\n");
   fprintf(fp, "PSEARCH_ALG     LOGARITHMIC\n");
   fprintf(fp, "BSEARCH_ALG     CROSS2\n");
   fprintf(fp, "IQSCALE         3\n");
   fprintf(fp, "PQSCALE         3\n");
   fprintf(fp, "BQSCALE         3\n");
   fprintf(fp, "REFERENCE_FRAME ORIGINAL\n");
   fprintf(fp, "FORCE_ENCODE_LAST_FRAME\n");
   fprintf(fp, "FRAME_RATE %d\n", MpegFrameRate);
   fflush(fp);
   fclose(fp);
   return mpeg_paramet;
}

static int
pnmstompg(char* mpeg_file_name, char* pnm_group, int pnm_num)
{
   char *mpeg_paramet;

   mpeg_paramet = create_paramet_file( mpeg_file_name, pnm_group, pnm_num);
   if( mpeg_encode(mpeg_paramet) == -1)
   {
      remove(mpeg_paramet);
      return 0;
   }
   else
   {
      remove(mpeg_paramet);
      return 1;
   }
}

static void 
clean_pnm_files(char* pnm_group, int num)
{
   char pnmfile[100];
   int i;

   for(i = 0; i < num; i++)
   {
     sprintf( pnmfile, "%s%d.pnm", pnm_group, i );
     remove(pnmfile);
  }
}

void image_expand(unsigned char *RGB240, unsigned char* new_RGB240)
{
   int i, j, k, m, ii, jj;
   float a[3], b[3], c[3], d[3];
   int RGB[3][4], newRGB[3];
   double x[4], y[4], xx[4], yy[4], new_x, new_y, r2;
   unsigned int p[4], ep;
   int num;
   double new_radius = mpeg_image_size/2.0;
   double ratio = (double)(mpeg_image_size-1)/(double)(image_size-1);

   for(j = 0; j < image_size-1; j++)
   {
       for(i = 0; i < image_size-1; i++)
      {
	 p[0] = 3*(i+j*image_size);  xx[0] = i*ratio; yy[0] = j*ratio;
	 p[1] = 3*(i+1+j*image_size);  xx[1] = (i+1)*ratio; yy[1] = j*ratio; 
	 p[2] = 3*(i+(j+1)*image_size);  xx[2] = i*ratio; yy[2] = (j+1)*ratio;
	 p[3] = 3*(i+1+(j+1)*image_size);  xx[3] = (i+1)*ratio; yy[3] = (j+1)*ratio;
	 
	 num = 0;
	 for(k = 0; k < 4; k++)
	 {
	    if(RGB240[p[k]+1] != 255)
	    {
	       RGB[0][num] = RGB240[p[k]];
	       RGB[1][num] = RGB240[p[k]+1];
	       RGB[2][num] = RGB240[p[k]+2];
	       x[num] = xx[k];
	       y[num] = yy[k];
	       num++;
	    }
	 }

	 for(m=0; m < 3; m++)
	 {
	    if (num==4)
	    {
	       a[m]= yy[2]*(RGB[m][1]-RGB[m][0])+yy[0]*(RGB[m][2]-RGB[m][3]);
	       a[m] /= ratio*ratio;
	       b[m]= xx[1]*(RGB[m][2]-RGB[m][0])+xx[0]*(RGB[m][1]-RGB[m][3]);
	       b[m] /= ratio*ratio;
	       c[m] = (RGB[m][0]+RGB[m][3]-RGB[m][1]-RGB[m][2])/(ratio*ratio);
	       d[m] = RGB[m][0] - a[m]*x[0] - b[m]*y[0] - c[m]*x[0]*y[0];
	    }
	 
	    else if (num==3)
	    {
	       a[m]=((RGB[m][1]-RGB[m][0])*(y[2]-y[0])
		  -(RGB[m][2]-RGB[m][0])*(y[1]-y[0]))
		   / ((x[1]-x[0])*(y[2]-y[0])-(x[2]-x[0])*(y[1]-y[0]));
	       b[m] = ((RGB[m][1]-RGB[m][0])*(x[2]-x[0])
		            -(RGB[m][2]-RGB[m][0])*(x[1]-x[0]))
		   / ((y[1]-y[0])*(x[2]-x[0])-(y[2]-y[0])*(x[1]-x[0]));
	       c[m]=0;
	       d[m] = RGB[m][0] - a[m]*x[0] - b[m]*y[0];
	    }
	    else if( num == 2)
	    {
	       if(fabs(y[0] - y[1]) < 1e-4)
	       {
		  a[m] = (RGB[m][1]-RGB[m][0])/(x[1]-x[0]);
		  b[m]=c[m] = 0;
		  d[m] = RGB[m][0] - a[m]*x[0];
	       }
	       else if(fabs(x[0] - x[1]) < 1e-4)
	       {
		  b[m] = (RGB[m][1]-RGB[m][0])/(y[1]-y[0]);
		  a[m]=c[m] = 0;
		  d[m] = RGB[m][0] - b[m]*y[0];
	       }
	       else if( (x[1]-x[0])*(y[1]-y[0]) > 1e-4 )
	       {
		  a[m] = b[m] = (RGB[m][1]-RGB[m][0])/(x[1]+y[1]-x[0]-y[0]);
		  c[m] = 0;
		  d[m] = RGB[m][0] - a[m]*(x[0] + y[0]);
	       }
	       else
	       {
		  a[m] = (RGB[m][1]-RGB[m][0])/(x[1]-y[1]-x[0]+y[0]);
		  b[m] = -a[m];
		  c[m] = 0;
		  d[m] = RGB[m][0] - a[m]*x[0] - b[m]*y[0];
	       }
	    }
	    else if( num == 1 )
	    {
	       a[m]=b[m]=c[m]= 0;
	       d[m]= RGB[m][0];
	    }
	 } 
	 for(jj = ceil(yy[1]); jj <= floor(yy[2]); jj++)
	 {
	    for(ii = ceil(xx[0]); ii <= floor(xx[1]); ii++)
	    {
	       ep = 3*(ii + mpeg_image_size*jj);

	       new_x = ii-(new_radius-0.5);
	       new_y = (new_radius-0.5)-jj;
	       r2 = new_x*new_x + new_y*new_y;

	       if(r2 > new_radius*new_radius)
	       {
		  new_RGB240[ep] = new_RGB240[ep+1] = new_RGB240[ep+2] = 255;
		  continue;
	       }
	       
	       for(m= 0; m < 3; m++)
	       {
		  newRGB[m] = rint(a[m]*ii + b[m]*jj + c[m]*ii*jj + d[m]);
		  if(newRGB[m] < 0) newRGB[m] = 0;
		  if(newRGB[m] >= 255) newRGB[m] = 254;
		  new_RGB240[ep+m]= newRGB[m];
	       }
	    }
	 }
      }
   }
}

