/*===========================================================================*
 * widget_set.c                                                              *
 *                                                                           *
 *      Procedures for packing several widgets                               *
 *                                                                           *
 *      GtkWidget* pack_spin (GtkWidget *parent, gchar *lbl_str,             *
 *                            gint label_width, gint spin_width,             *
 *                            gfloat dflt, gfloat min, gfloat max,           *
 *                            gfloat step, gfloat pstep, gfloat digit )      *
 *      GtkWidget* pack_scale(GtkWidget *parent, gchar *label,               *
 *                            gint label_width, gint scale_width,            *
 *                            gfloat deflt, gfloat min, gfloat max,          *
 *                            gfloat step, gfloat pstep )                    *
 *      GtkWidget* pack_radio_button(GtkWidget *parent, gchar *label,        * 
 *                            GSList **group, gint padding)                  *
 *      GtkWidget* pack_button(GtkWidget *parent, gchar *label)              *
 *      GtkWidget* pack_label(GtkWidget *parent, gchar *str, gint padding)   *
 *                                                                           *
 *                                                            (by A. Miyake) *
 *===========================================================================*/
#include <gtk/gtk.h>

GtkWidget* pack_spin (GtkWidget *parent, gchar *lbl_str, gint label_width, 
			 gint spin_width, gfloat dflt, gfloat min, gfloat max, 
			 gfloat step, gfloat pstep, gfloat digit )
{
   
   GtkWidget *box, *label, *spin;
   GtkObject *adj;

   box = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (parent), box, FALSE, FALSE, 2);
  
   label = gtk_label_new(lbl_str);
   gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
   gtk_widget_set_usize (label, label_width, 0);
   gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);

   adj = gtk_adjustment_new (dflt, min, max, step, pstep, pstep);
   spin = gtk_spin_button_new( GTK_ADJUSTMENT(adj), 0, digit);

   gtk_widget_set_usize (spin, spin_width, 0);
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON(spin), TRUE);
   gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON(spin), TRUE);
   gtk_spin_button_set_shadow_type (GTK_SPIN_BUTTON(spin), GTK_SHADOW_IN);

   gtk_box_pack_start (GTK_BOX (box), spin, FALSE, FALSE, 0);

   gtk_object_set_data(GTK_OBJECT(spin), "label", label);
   return spin;
}

GtkWidget* pack_scale (GtkWidget *parent, gchar *lbl_str, gint label_width, 
	      gint scale_width, gfloat dflt, gfloat min, gfloat max, 
	      gfloat step, gfloat pstep )
{
   GtkWidget *box, *label, *scale;
   GtkObject *adj;

   box = gtk_hbox_new (FALSE, 10);
   gtk_box_pack_start (GTK_BOX (parent), box, FALSE, FALSE, 5);
  
   label = gtk_label_new(lbl_str);
   gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
   gtk_widget_set_usize (label, label_width, 0);
   gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);

   adj = gtk_adjustment_new (dflt, min, max, step, pstep, pstep);
   scale = gtk_hscale_new( GTK_ADJUSTMENT(adj));
   gtk_range_set_update_policy(GTK_RANGE(scale),GTK_UPDATE_CONTINUOUS);  
   gtk_widget_set_usize (scale, scale_width, 0);
   gtk_box_pack_start (GTK_BOX (box), scale, FALSE, FALSE, 0);

   return scale;
}

GtkWidget* pack_radio_button(GtkWidget *parent, gchar *label,
				     GSList **group, gint padding)
{
   GtkWidget* button = gtk_radio_button_new_with_label(*group, label);
   *group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
   gtk_box_pack_start(GTK_BOX (parent), button, FALSE, FALSE, padding);
   
   return button;
}


GtkWidget*  
pack_button(GtkWidget *parent, gchar *label)
{
   GtkWidget* button = gtk_button_new_with_label (label);
   gtk_container_set_border_width (GTK_CONTAINER (button), 5);
   gtk_box_pack_start(GTK_BOX(parent), button, TRUE, TRUE, 0);
   return button;
}
   
GtkWidget*  
pack_label(GtkWidget *parent, gchar *str, gint padding)
{
   GtkWidget* label = gtk_label_new(str);
   gtk_box_pack_start(GTK_BOX (parent), label, FALSE, FALSE, padding);
   return label;
}

